@@ -3,9 +3,9 @@
 #' Generate an Access Token for the Adobe Analytics v2.0 API
 #'
 #' **Note:** `aw_auth()` is the primary function used for authorization. `auth_oauth()`
-#' and `auth_jwt()` should typically not be called directly.
+#' and `auth_s2s()` should typically not be called directly.
 #'
-#' @param type Either 'jwt' or 'oauth'. This can be set explicitly, but a best practice is
+#' @param type Either 's2s' or 'oauth'. This can be set explicitly, but a best practice is
 #' to run `aw_auth_with()` to set the authorization type as an environment variable before
 #' running `aw_auth()`
 #' @param ... Additional arguments passed to auth functions.
@@ -18,6 +18,7 @@
 #' @param private_key Filename of the private key for JWT authentication.
 #' @param jwt_token _(Optional)_ A custom, encoded, signed JWT claim. If used,
 #'   `client_id` and `client_secret` are still required.
+#' @param s2s_token _(Optional)_ A custom, encoded, S2S authentication token.
 #' @param use_oob if `FALSE`, use a local webserver for the OAuth dance.
 #'   Otherwise, provide a URL to the user and prompt for a validation code.
 #'   Defaults to the value of the `httr_oob_default` default, or TRUE if
@@ -27,18 +28,21 @@
 #'
 #' @return The path of the cached token. This is returned invisibly.
 #' @family auth
-#' @aliases aw_auth auth_jwt auth_oauth
+#' @aliases aw_auth auth_jwt auth_oauth auth_s2s
 #' @export
 aw_auth <- function(type = aw_auth_with(), ...) {
 
   if (is.null(type)) {
     stop("Authentication type missing, please set an auth type with `aw_auth_with`")
   }
-  type <- match.arg(type, c("jwt", "oauth"))
+  type <- match.arg(type, c("s2s", "oauth", "jwt"))
+
+  aw_auth_with(type)
 
   switch(type,
          jwt = auth_jwt(...),
-         oauth = auth_oauth(...)
+         oauth = auth_oauth(...),
+         s2s = auth_s2s(...)
   )
 }
 
@@ -53,7 +57,7 @@ aw_auth <- function(type = aw_auth_with(), ...) {
 #' `aw_auth_with` sets the type of authorization for the session. This is used
 #' as the default by `aw_auth()` when no specific option is given.
 #'
-#' @param type The authorization type: 'oauth' or 'jwt'
+#' @param type The authorization type: 'oauth', 's2s', or 'jwt'
 #' @param path The location for the cached authorization token. It should be a
 #' directory, rather than a filename. If this option is not set, the current
 #' working directory is used instead. If the location does not exist, it will
@@ -72,7 +76,7 @@ aw_auth_with <- function(type) {
     if (missing(type)) return(getOption("adobeanalyticsr.auth_type"))
 
     if (!is.null(type)) {
-        type <- match.arg(type, c("oauth", "jwt"))
+        type <- match.arg(type, c("oauth", "s2s", "jwt"))
     }
 
     options(adobeanalyticsr.auth_type = type)
@@ -134,16 +138,21 @@ retrieve_aw_token <- function(...) {
         cached_token_exists <- file.exists(path)
 
         if (cached_token_exists && type == "oauth") {
-            message(paste("Retrieving cached token:", path))
-            token <- readRDS(path)
-            type <- token_type(token)
+          message(paste("Retrieving cached token:", path))
+          token <- readRDS(path)
+          type <- token_type(token)
 
-            .adobeanalytics$token <- token
+          .adobeanalytics$token <- token
+        } else if (cached_token_exists && type == 'jwt'){
+          message("No JWT session token or cached token -- generating new token")
+          aw_auth(type = aw_auth_with(), ...)
+          token <- .adobeanalytics$token
+          type <- aw_auth_with()
         } else {
-            message("No session token or cached token -- generating new token")
-            aw_auth(type = aw_auth_with(), ...)
-            token <- .adobeanalytics$token
-            type <- aw_auth_with()
+          message("No S2S session token or cached token -- generating new token")
+          aw_auth(type = aw_auth_with(), ...)
+          token <- .adobeanalytics$token
+          type <- aw_auth_with()
         }
     }
 
@@ -152,8 +161,13 @@ retrieve_aw_token <- function(...) {
         if (!token$validate()) {
         # This might be the wrong thing to do with OAuth, but it's the right
         # thing to do for JWT
+
         .adobeanalytics$token$refresh()
         }
+    }else if(type == 's2s') {
+      if (!token$validate()) {
+        .adobeanalytics$token$refresh()
+      }
     }
 
     return(.adobeanalytics$token)
@@ -182,14 +196,16 @@ token_path <- function(...) {
 #'
 #' @param token An `httr` `reponse` object or `oauth2.0_token` object
 #'
-#' @return Either 'oauth' or 'jwt'
+#' @return Either 'oauth' or 's2s'
 #'
 #' @noRd
 token_type <- function(token) {
     if (inherits(token, "Token2.0")) {
-        "oauth"
+      "oauth"
     } else if (inherits(token, "AdobeJwtToken")) {
-        "jwt"
+      "jwt"
+    } else if (inherits(token, "AdobeS2SToken")) {
+      's2s'
     } else if (is.null(token)) {
         NULL
     } else {
@@ -218,6 +234,7 @@ get_token_config <- function(client_id,
     switch(type,
         oauth = httr::config(token = token),
         jwt = httr::add_headers(Authorization = paste("Bearer", content(token$token)$access_token)),
+        s2s = httr::add_headers(Authorization = paste("Bearer", token$token$access)),
         stop("Unknown token type")
     )
 }
@@ -234,7 +251,8 @@ get_token_config <- function(client_id,
 #'
 #' @return List of length two with elements `client_id` and `client_secret`
 #' @noRd
-get_env_vars <- function() {
+get_env_vars <- function(type = aw_auth_with()) {
+  if(type == 'oauth' || type == 'jwt') {
     client_id <- .adobeanalytics$client_id
     client_secret <- .adobeanalytics$client_secret
 
@@ -252,11 +270,24 @@ get_env_vars <- function() {
         stop("Cannot automatically authenticate due to missing environment variables: ", paste(missing_envs, collapse = ", "),
              call. = FALSE)
     }
-
     list(
-        client_id = client_id,
-        client_secret = client_secret
+      client_id = client_id,
+      client_secret = client_secret
     )
+  } else if(type == 's2s') {
+    client_id <- .adobeanalytics$client_id
+    client_secret <- .adobeanalytics$client_secret
+
+    if (is.null(client_id) | is.null(client_secret)) {
+      secrets <- jsonlite::fromJSON(Sys.getenv("AW_AUTH_FILE"))
+      client_id <- secrets$CLIENT_ID
+      client_secret <-  secrets$CLIENT_SECRETS
+    }
+    list(
+      client_id = client_id,
+      client_secret = client_secret
+    )
+  }
 }
 
 
@@ -277,6 +308,8 @@ auth_jwt <- function(file = Sys.getenv("AW_AUTH_FILE"),
     stop("Variable 'AW_AUTH_FILE' not found but required for default JWT authentication.\nSee `?aw_auth`")
   }
 
+  aw_auth_with('jwt')
+
   secrets <- jsonlite::fromJSON(file)
 
   resp <- auth_jwt_gen(secrets = secrets, private_key = private_key, jwt_token = jwt_token)
@@ -303,14 +336,14 @@ auth_jwt_gen <- function(secrets,
                          private_key,
                          jwt_token = NULL) {
     # Look for missing elements of secrets
-    required_but_missing <- setdiff(c("API_KEY", 
-                                      "CLIENT_SECRET", 
+    required_but_missing <- setdiff(c("API_KEY",
+                                      "CLIENT_SECRET",
                                       "ORG_ID",
                                       "TECHNICAL_ACCOUNT_ID"),
                                     names(secrets))
 
     if (length(required_but_missing) != 0) {
-        stop("Did not find required variables in auth file: ", 
+        stop("Did not find required variables in auth file: ",
              paste(required_but_missing, collapse = ", "))
     }
 
@@ -423,6 +456,129 @@ AdobeJwtToken <- R6::R6Class("AdobeJwtToken", list(
     }
 ))
 
+# S2S ------------------------------------------------------------------
+
+#' @family auth
+#' @describeIn aw_auth Authenticate with S2S token
+#' @export
+auth_s2s <- function(file = Sys.getenv("AW_AUTH_FILE"),
+                     s2s_token = NULL,
+                     type = 's2s',
+                     ...) {
+  if (file == "") {
+    stop("Variable 'AW_AUTH_FILE' not found but required for default S2S authentication.\nSee `?aw_auth`")
+  }
+
+  secrets <- jsonlite::fromJSON(file)
+
+  aw_auth_with('s2s')
+
+  resp <- auth_s2s_gen(secrets = secrets, s2s_token = s2s_token)
+
+  # If successful
+  message("Successfully authenticated with S2S: access token valid until ",
+          as.POSIXct(resp$expires_at, origin = "1970-01-01"))
+
+  .adobeanalytics$token <- AdobeS2SToken$new(resp, secrets)
+  .adobeanalytics$client_id <- secrets$CLIENT_ID
+  .adobeanalytics$client_secret <- secrets$CLIENT_SECRETS
+}
+
+
+#' Generate the authorization response object for S2S
+#'
+#' @param secrets List of secret values, see `auth_s2s`
+#' @param s2s_token Optional, a  S2S token (e.g., a cached token)
+#'
+#' @noRd
+auth_s2s_gen <- function(secrets,
+                         s2s_token = NULL) {
+    # Look for missing elements of secrets
+  stopifnot(is.character(secrets$CLIENT_ID))
+  stopifnot(is.character(secrets$CLIENT_SECRETS))
+  stopifnot(is.character(secrets$SCOPES))
+
+  if (any(c(secrets$CLIENT_ID, secrets$CLIENT_SECRETS) == "")) {
+    stop("Client ID, Client Secrets, or Scopes not found in Auth File.")
+  }
+
+
+    token <- get_s2s_token(s2s_token = s2s_token,
+                               client_id = secrets$CLIENT_ID,
+                               scopes = secrets$SCOPES,
+                               client_secrets = secrets$CLIENT_SECRETS
+                               )
+
+    token
+}
+
+
+#' Get an encoded, signed S2S token
+#'
+#' Gets a S2S token
+#'
+#' @param s2s_token Optional, a S2S token
+#' @param client_id Client ID
+#' @param client_secrets Secrets, can be more than one, a vector if more than one or a string if only 1
+#' @param scopes Scopes as listed in the project
+#'
+#' @return A S2S token generated by httr2 oauth_flow_client_credentials
+#' @noRd
+get_s2s_token <- function(s2s_token = NULL,
+                          client_id,
+                          client_secrets,
+                          scopes = c("openid","AdobeID","additional_info.projectedProductContext","read_pc.acp","read_pc","read_pc.dma_tartan","additional_info","read_organizations","session")
+                          ) {
+    if (is.null(s2s_token)) {
+      s2s_client <- httr2::oauth_client(client_id,
+                                        'https://ims-na1.adobelogin.com/ims/token/v3',
+                                        secret = paste0(client_secrets, collapse = ','),
+                                        auth = 'body')
+
+      s2s_token <- httr2::oauth_flow_client_credentials(client = s2s_client,
+                                                        scope = paste0(scopes, collapse = ','),
+                                                        token_params = c(grant_type = 'client_credentials',
+                                                                         client_id = client_id))
+    }
+
+    s2s_token
+}
+
+
+#' Adobe S2S token response
+#'
+#' Includes the response object containing the bearer token as well as the
+#' credentials used to generate the token for seamless refreshing.
+#'
+#'
+#' @section Methods:
+#' * `refresh()`: refresh access token (if possible)
+#' * `validate()`: TRUE if the token is still valid, FALSE otherwise
+#'
+#' @docType class
+#' @keywords internal
+#' @format An R6 object
+#' @importFrom R6 R6Class
+#' @noRd
+AdobeS2SToken <- R6::R6Class("AdobeS2SToken", list(
+    secrets = NULL,
+    token = NULL,
+    initialize = function(token, secrets) {
+        self$secrets <- secrets
+        self$token <- token
+    },
+    can_refresh = function() {
+        FALSE
+    },
+    refresh = function() {
+        self$token <- auth_s2s_gen(self$secrets)
+        self
+    },
+    validate = function() {
+        self$token$expires_at + self$token$expires_at / 1000 > Sys.time() - 1200
+    }
+))
+
 
 # OAuth ----------------------------------------------------------------
 #' @family auth
@@ -445,6 +601,8 @@ auth_oauth <- function(client_id = Sys.getenv("AW_CLIENT_ID"),
         stop("Client ID or Client Secret not found. Are your environment variables named `AW_CLIENT_ID` and `AW_CLIENT_SECRET`?")
     }
 
+    aw_auth_with('oauth')
+
     aw_endpoint <- httr::oauth_endpoint(
         authorize = "authorize/v2/",
         access = "token/v3",
