@@ -22,6 +22,7 @@
 #'
 aw_call_api <- function(req_path,
                         body = NULL,
+                        content_type = NULL,
                         debug = FALSE,
                         company_id) {
     assertthat::assert_that(
@@ -29,7 +30,6 @@ aw_call_api <- function(req_path,
         assertthat::is.string(company_id)
     )
 
-
     env_vars <- get_env_vars()
     token_config <- get_token_config(client_id = env_vars$client_id,
                                      client_secret = env_vars$client_secret)
@@ -50,6 +50,7 @@ aw_call_api <- function(req_path,
                        token_config,
                        debug_call,
                        httr::add_headers(
+                          `Content-type` = content_type,
                            `x-api-key` = env_vars$client_id,
                            `x-proxy-global-company-id` = company_id
                        ))
