@@ -3,4 +3,4 @@ utils::globalVariables(c("itemId", "value", "metric", "data", "name",
                          'dataUpperBound', 'day', 'eventnm', 'qickview',
                          'columnId', 'filters', 'filtername', 'mfinalname',
                          'pred_items', 'pred_list', 'segmentIds', 'metrics',
-                         'components', 'status'))
+                         'components', 'status', 'id'))
